@@ -75,11 +75,27 @@ class GitlabTransferProjectError(GitlabOperationError):
     pass
 
 
-class GitlabBuildCancelError(GitlabOperationError):
+class GitlabCancelError(GitlabOperationError):
     pass
 
 
-class GitlabBuildRetryError(GitlabOperationError):
+class GitlabBuildCancelError(GitlabCancelError):
+    pass
+
+
+class GitlabPipelineCancelError(GitlabCancelError):
+    pass
+
+
+class GitlabRetryError(GitlabOperationError):
+    pass
+
+
+class GitlabBuildRetryError(GitlabRetryError):
+    pass
+
+
+class GitlabPipelineRetryError(GitlabRetryError):
     pass
 
 
@@ -111,6 +127,10 @@ class GitlabMROnBuildSuccessError(GitlabOperationError):
     pass
 
 
+class GitlabTodoError(GitlabOperationError):
+    pass
+
+
 def raise_error_from_response(response, error, expected_code=200):
     """Tries to parse gitlab error message from response and raises error.
 
