@@ -168,8 +168,6 @@ class GitlabObject(object):
     _id_in_update_url = True
     _constructorTypes = None
 
-    #: Whether _get_list_or_object should return list or object when id is None
-    getListWhenNoId = True
     #: Tells if GitLab-api allows retrieving single objects.
     canGet = True
     #: Tells if GitLab-api allows listing of objects.
@@ -209,7 +207,8 @@ class GitlabObject(object):
     #: Attribute to use as ID when displaying the object.
     shortPrintAttr = None
 
-    def _data_for_gitlab(self, extra_parameters={}, update=False):
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
         data = {}
         if update and (self.requiredUpdateAttrs or self.optionalUpdateAttrs):
             attributes = itertools.chain(self.requiredUpdateAttrs,
@@ -227,7 +226,7 @@ class GitlabObject(object):
 
         data.update(extra_parameters)
 
-        return json.dumps(data)
+        return json.dumps(data) if as_json else data
 
     @classmethod
     def list(cls, gl, **kwargs):
@@ -281,13 +280,6 @@ class GitlabObject(object):
 
             raise GitlabGetError("Object not found")
 
-    @classmethod
-    def _get_list_or_object(cls, gl, id, **kwargs):
-        if id is None and cls.getListWhenNoId:
-            return cls.list(gl, **kwargs)
-        else:
-            return cls.get(gl, id, **kwargs)
-
     def _get_object(self, k, v):
         if self._constructorTypes and k in self._constructorTypes:
             return globals()[self._constructorTypes[k]](self.gitlab, v)
@@ -295,6 +287,9 @@ class GitlabObject(object):
             return v
 
     def _set_from_dict(self, data):
+        if not hasattr(data, 'items'):
+            return
+
         for k, v in data.items():
             if isinstance(v, list):
                 self.__dict__[k] = []
@@ -495,6 +490,42 @@ class GitlabObject(object):
         return not self.__eq__(other)
 
 
+class SidekiqManager(object):
+    """Manager for the Sidekiq methods.
+
+    This manager doesn't actually manage objects but provides helper fonction
+    for the sidekiq metrics API.
+    """
+    def __init__(self, gl):
+        """Constructs a Sidekiq manager.
+
+        Args:
+            gl (gitlab.Gitlab): Gitlab object referencing the GitLab server.
+        """
+        self.gitlab = gl
+
+    def _simple_get(self, url, **kwargs):
+        r = self.gitlab._raw_get(url, **kwargs)
+        raise_error_from_response(r, GitlabGetError)
+        return r.json()
+
+    def queue_metrics(self, **kwargs):
+        """Returns the registred queues information."""
+        return self._simple_get('/sidekiq/queue_metrics', **kwargs)
+
+    def process_metrics(self, **kwargs):
+        """Returns the registred sidekiq workers."""
+        return self._simple_get('/sidekiq/process_metrics', **kwargs)
+
+    def job_stats(self, **kwargs):
+        """Returns statistics about the jobs performed."""
+        return self._simple_get('/sidekiq/job_stats', **kwargs)
+
+    def compound_metrics(self, **kwargs):
+        """Returns all available metrics and statistics."""
+        return self._simple_get('/sidekiq/compound_metrics', **kwargs)
+
+
 class UserEmail(GitlabObject):
     _url = '/users/%(user_id)s/emails'
     canUpdate = False
@@ -537,18 +568,12 @@ class User(GitlabObject):
         ('keys', UserKeyManager, [('user_id', 'id')])
     ]
 
-    def _data_for_gitlab(self, extra_parameters={}, update=False):
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
         if hasattr(self, 'confirm'):
             self.confirm = str(self.confirm).lower()
         return super(User, self)._data_for_gitlab(extra_parameters)
 
-    def Key(self, id=None, **kwargs):
-        warnings.warn("`Key` is deprecated, use `keys` instead",
-                      DeprecationWarning)
-        return UserKey._get_list_or_object(self.gitlab, id,
-                                           user_id=self.id,
-                                           **kwargs)
-
     def block(self, **kwargs):
         """Blocks the user."""
         url = '/users/%s/block' % self.id
@@ -651,23 +676,30 @@ class CurrentUser(GitlabObject):
         ('keys', CurrentUserKeyManager, [('user_id', 'id')])
     ]
 
-    def Key(self, id=None, **kwargs):
-        warnings.warn("`Key` is deprecated, use `keys` instead",
-                      DeprecationWarning)
-        return CurrentUserKey._get_list_or_object(self.gitlab, id, **kwargs)
-
 
 class ApplicationSettings(GitlabObject):
     _url = '/application/settings'
     _id_in_update_url = False
-    optionalUpdateAttrs = ['after_sign_out_path', 'default_branch_protection',
+    optionalUpdateAttrs = ['after_sign_out_path',
+                           'container_registry_token_expire_delay',
+                           'default_branch_protection',
                            'default_project_visibility',
                            'default_projects_limit',
-                           'default_snippet_visibility', 'gravatar_enabled',
-                           'home_page_url', 'restricted_signup_domains',
+                           'default_snippet_visibility',
+                           'domain_blacklist',
+                           'domain_blacklist_enabled',
+                           'domain_whitelist',
+                           'enabled_git_access_protocol'
+                           'gravatar_enabled',
+                           'home_page_url',
+                           'max_attachment_size',
+                           'repository_storage',
+                           'restricted_signup_domains',
                            'restricted_visibility_levels',
-                           'session_expire_delay', 'sign_in_text',
-                           'signin_enabled', 'signup_enabled',
+                           'session_expire_delay',
+                           'sign_in_text',
+                           'signin_enabled',
+                           'signup_enabled',
                            'twitter_sharing_enabled',
                            'user_oauth_applications']
     canList = False
@@ -736,6 +768,34 @@ class GroupProjectManager(BaseManager):
     obj_cls = GroupProject
 
 
+class GroupAccessRequest(GitlabObject):
+    _url = '/groups/%(group_id)s/access_requests'
+    canGet = 'from_list'
+    canUpdate = False
+
+    def approve(self, access_level=gitlab.DEVELOPER_ACCESS, **kwargs):
+        """Approve an access request.
+
+        Attrs:
+            access_level (int): The access level for the user.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabUpdateError: If the server fails to perform the request.
+        """
+
+        url = ('/groups/%(group_id)s/access_requests/%(id)s/approve' %
+               {'group_id': self.group_id, 'id': self.id})
+        data = {'access_level': access_level}
+        r = self.gitlab._raw_put(url, data=data, **kwargs)
+        raise_error_from_response(r, GitlabUpdateError, 201)
+        self._set_from_dict(r.json())
+
+
+class GroupAccessRequestManager(BaseManager):
+    obj_cls = GroupAccessRequest
+
+
 class Group(GitlabObject):
     _url = '/groups'
     _constructorTypes = {'projects': 'Project'}
@@ -743,26 +803,22 @@ class Group(GitlabObject):
     optionalCreateAttrs = ['description', 'visibility_level']
     optionalUpdateAttrs = ['name', 'path', 'description', 'visibility_level']
     shortPrintAttr = 'name'
-    managers = [('members', GroupMemberManager, [('group_id', 'id')]),
-                ('projects', GroupProjectManager, [('group_id', 'id')]),
-                ('issues', GroupIssueManager, [('group_id', 'id')])]
+    managers = [
+        ('accessrequests', GroupAccessRequestManager, [('group_id', 'id')]),
+        ('members', GroupMemberManager, [('group_id', 'id')]),
+        ('projects', GroupProjectManager, [('group_id', 'id')]),
+        ('issues', GroupIssueManager, [('group_id', 'id')])
+    ]
 
-    GUEST_ACCESS = 10
-    REPORTER_ACCESS = 20
-    DEVELOPER_ACCESS = 30
-    MASTER_ACCESS = 40
-    OWNER_ACCESS = 50
+    GUEST_ACCESS = gitlab.GUEST_ACCESS
+    REPORTER_ACCESS = gitlab.REPORTER_ACCESS
+    DEVELOPER_ACCESS = gitlab.DEVELOPER_ACCESS
+    MASTER_ACCESS = gitlab.MASTER_ACCESS
+    OWNER_ACCESS = gitlab.OWNER_ACCESS
 
-    VISIBILITY_PRIVATE = 0
-    VISIBILITY_INTERNAL = 10
-    VISIBILITY_PUBLIC = 20
-
-    def Member(self, id=None, **kwargs):
-        warnings.warn("`Member` is deprecated, use `members` instead",
-                      DeprecationWarning)
-        return GroupMember._get_list_or_object(self.gitlab, id,
-                                               group_id=self.id,
-                                               **kwargs)
+    VISIBILITY_PRIVATE = gitlab.VISIBILITY_PRIVATE
+    VISIBILITY_INTERNAL = gitlab.VISIBILITY_INTERNAL
+    VISIBILITY_PUBLIC = gitlab.VISIBILITY_PUBLIC
 
     def transfer_project(self, id, **kwargs):
         """Transfers a project to this new groups.
@@ -988,7 +1044,7 @@ class ProjectCommitStatusManager(BaseManager):
 class ProjectCommitComment(GitlabObject):
     _url = '/projects/%(project_id)s/repository/commits/%(commit_id)s/comments'
     canUpdate = False
-    cantGet = False
+    canGet = False
     canDelete = False
     requiredUrlAttrs = ['project_id', 'commit_id']
     requiredCreateAttrs = ['note']
@@ -1162,7 +1218,8 @@ class ProjectIssue(GitlabObject):
     managers = [('notes', ProjectIssueNoteManager,
                  [('project_id', 'project_id'), ('issue_id', 'id')])]
 
-    def _data_for_gitlab(self, extra_parameters={}, update=False):
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
         # Gitlab-api returns labels in a json list and takes them in a
         # comma separated list.
         if hasattr(self, "labels"):
@@ -1174,14 +1231,6 @@ class ProjectIssue(GitlabObject):
         return super(ProjectIssue, self)._data_for_gitlab(extra_parameters,
                                                           update)
 
-    def Note(self, id=None, **kwargs):
-        warnings.warn("`Note` is deprecated, use `notes` instead",
-                      DeprecationWarning)
-        return ProjectIssueNote._get_list_or_object(self.gitlab, id,
-                                                    project_id=self.project_id,
-                                                    issue_id=self.id,
-                                                    **kwargs)
-
     def subscribe(self, **kwargs):
         """Subscribe to an issue.
 
@@ -1225,6 +1274,17 @@ class ProjectIssue(GitlabObject):
         raise_error_from_response(r, GitlabUpdateError, 201)
         self._set_from_dict(r.json())
 
+    def todo(self, **kwargs):
+        """Create a todo for the issue.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+        """
+        url = ('/projects/%(project_id)s/issues/%(issue_id)s/todo' %
+               {'project_id': self.project_id, 'issue_id': self.id})
+        r = self.gitlab._raw_post(url, **kwargs)
+        raise_error_from_response(r, GitlabTodoError, [201, 304])
+
 
 class ProjectIssueManager(BaseManager):
     obj_cls = ProjectIssue
@@ -1264,10 +1324,6 @@ class ProjectTagRelease(GitlabObject):
     shortPrintAttr = 'description'
 
 
-class ProjectTagReleaseManager(BaseManager):
-    obj_cls = ProjectTagRelease
-
-
 class ProjectTag(GitlabObject):
     _url = '/projects/%(project_id)s/repository/tags'
     _constructorTypes = {'release': 'ProjectTagRelease',
@@ -1337,25 +1393,16 @@ class ProjectMergeRequest(GitlabObject):
     managers = [('notes', ProjectMergeRequestNoteManager,
                  [('project_id', 'project_id'), ('merge_request_id', 'id')])]
 
-    def Note(self, id=None, **kwargs):
-        warnings.warn("`Note` is deprecated, use `notes` instead",
-                      DeprecationWarning)
-        return ProjectMergeRequestNote._get_list_or_object(
-            self.gitlab, id, project_id=self.project_id,
-            merge_request_id=self.id, **kwargs)
-
-    def _data_for_gitlab(self, extra_parameters={}, update=False):
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
         data = (super(ProjectMergeRequest, self)
-                ._data_for_gitlab(extra_parameters, update=update))
+                ._data_for_gitlab(extra_parameters, update=update,
+                                  as_json=False))
         if update:
             # Drop source_branch attribute as it is not accepted by the gitlab
             # server (Issue #76)
-            # We need to unserialize and reserialize the
-            # data, this is far from optimal
-            d = json.loads(data)
-            d.pop('source_branch', None)
-            data = json.dumps(d)
-        return data
+            data.pop('source_branch', None)
+        return json.dumps(data)
 
     def subscribe(self, **kwargs):
         """Subscribe to a MR.
@@ -1486,6 +1533,17 @@ class ProjectMergeRequest(GitlabObject):
         raise_error_from_response(r, errors)
         self._set_from_dict(r.json())
 
+    def todo(self, **kwargs):
+        """Create a todo for the merge request.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+        """
+        url = ('/projects/%(project_id)s/merge_requests/%(mr_id)s/todo' %
+               {'project_id': self.project_id, 'mr_id': self.id})
+        r = self.gitlab._raw_post(url, **kwargs)
+        raise_error_from_response(r, GitlabTodoError, [201, 304])
+
 
 class ProjectMergeRequestManager(BaseManager):
     obj_cls = ProjectMergeRequest
@@ -1567,7 +1625,6 @@ class ProjectFile(GitlabObject):
                            'commit_message']
     optionalCreateAttrs = ['encoding']
     requiredDeleteAttrs = ['branch_name', 'commit_message', 'file_path']
-    getListWhenNoId = False
     shortPrintAttr = 'file_path'
     getRequiresId = False
 
@@ -1584,6 +1641,43 @@ class ProjectFileManager(BaseManager):
     obj_cls = ProjectFile
 
 
+class ProjectPipeline(GitlabObject):
+    _url = '/projects/%(project_id)s/pipelines'
+    canCreate = False
+    canUpdate = False
+    canDelete = False
+
+    def retry(self, **kwargs):
+        """Retries failed builds in a pipeline.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabPipelineRetryError: If the retry cannot be done.
+        """
+        url = ('/projects/%(project_id)s/pipelines/%(id)s/retry' %
+               {'project_id': self.project_id, 'id': self.id})
+        r = self.gitlab._raw_post(url, data=None, content_type=None, **kwargs)
+        raise_error_from_response(r, GitlabPipelineRetryError, 201)
+        self._set_from_dict(r.json())
+
+    def cancel(self, **kwargs):
+        """Cancel builds in a pipeline.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabPipelineCancelError: If the retry cannot be done.
+        """
+        url = ('/projects/%(project_id)s/pipelines/%(id)s/cancel' %
+               {'project_id': self.project_id, 'id': self.id})
+        r = self.gitlab._raw_post(url, data=None, content_type=None, **kwargs)
+        raise_error_from_response(r, GitlabPipelineRetryError, 200)
+        self._set_from_dict(r.json())
+
+
+class ProjectPipelineManager(BaseManager):
+    obj_cls = ProjectPipeline
+
+
 class ProjectSnippetNote(GitlabObject):
     _url = '/projects/%(project_id)s/snippets/%(snippet_id)s/notes'
     _constructorTypes = {'author': 'User'}
@@ -1613,15 +1707,6 @@ class ProjectSnippet(GitlabObject):
                       DeprecationWarning)
         return self.content()
 
-    def Note(self, id=None, **kwargs):
-        warnings.warn("`Note` is deprecated, use `notes` instead",
-                      DeprecationWarning)
-        return ProjectSnippetNote._get_list_or_object(
-            self.gitlab, id,
-            project_id=self.project_id,
-            snippet_id=self.id,
-            **kwargs)
-
     def content(self, streamed=False, action=None, chunk_size=1024, **kwargs):
         """Return the raw content of a snippet.
 
@@ -1673,6 +1758,121 @@ class ProjectVariableManager(BaseManager):
     obj_cls = ProjectVariable
 
 
+class ProjectService(GitlabObject):
+    _url = '/projects/%(project_id)s/services/%(service_name)s'
+    canList = False
+    canCreate = False
+    _id_in_update_url = False
+    _id_in_delete_url = False
+    requiredUrlAttrs = ['project_id', 'service_name']
+
+    _service_attrs = {
+        'asana': (('api_key', ), ('restrict_to_branch', )),
+        'assembla': (('token', ), ('subdomain', )),
+        'bamboo': (('bamboo_url', 'build_key', 'username', 'password'),
+                   tuple()),
+        'buildkite': (('token', 'project_url'), ('enable_ssl_verification', )),
+        'campfire': (('token', ), ('subdomain', 'room')),
+        'custom-issue-tracker': (('new_issue_url', 'issues_url',
+                                  'project_url'),
+                                 ('description', 'title')),
+        'drone-ci': (('token', 'drone_url'), ('enable_ssl_verification', )),
+        'emails-on-push': (('recipients', ), ('disable_diffs',
+                                              'send_from_committer_email')),
+        'external-wiki': (('external_wiki_url', ), tuple()),
+        'flowdock': (('token', ), tuple()),
+        'gemnasium': (('api_key', 'token', ), tuple()),
+        'hipchat': (('token', ), ('color', 'notify', 'room', 'api_version',
+                                  'server')),
+        'irker': (('recipients', ), ('default_irc_uri', 'server_port',
+                                     'server_host', 'colorize_messages')),
+        'jira': (('new_issue_url', 'project_url', 'issues_url'),
+                 ('description', 'username', 'password')),
+        'pivotaltracker': (('token', ), tuple()),
+        'pushover': (('api_key', 'user_key', 'priority'), ('device', 'sound')),
+        'redmine': (('new_issue_url', 'project_url', 'issues_url'),
+                    ('description', )),
+        'slack': (('webhook', ), ('username', 'channel')),
+        'teamcity': (('teamcity_url', 'build_type', 'username', 'password'),
+                     tuple())
+    }
+
+    def _data_for_gitlab(self, extra_parameters={}, update=False,
+                         as_json=True):
+        data = (super(ProjectService, self)
+                ._data_for_gitlab(extra_parameters, update=update,
+                                  as_json=False))
+        missing = []
+        # Mandatory args
+        for attr in self._service_attrs[self.service_name][0]:
+            if not hasattr(self, attr):
+                missing.append(attr)
+            else:
+                data[attr] = getattr(self, attr)
+
+        if missing:
+            raise GitlabUpdateError('Missing attribute(s): %s' %
+                                    ", ".join(missing))
+
+        # Optional args
+        for attr in self._service_attrs[self.service_name][1]:
+            if hasattr(self, attr):
+                data[attr] = getattr(self, attr)
+
+        return json.dumps(data)
+
+
+class ProjectServiceManager(BaseManager):
+    obj_cls = ProjectService
+
+    def available(self, **kwargs):
+        """List the services known by python-gitlab.
+
+        Returns:
+            list (str): The list of service code names.
+        """
+        return json.dumps(ProjectService._service_attrs.keys())
+
+
+class ProjectAccessRequest(GitlabObject):
+    _url = '/projects/%(project_id)s/access_requests'
+    canGet = 'from_list'
+    canUpdate = False
+
+    def approve(self, access_level=gitlab.DEVELOPER_ACCESS, **kwargs):
+        """Approve an access request.
+
+        Attrs:
+            access_level (int): The access level for the user.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabUpdateError: If the server fails to perform the request.
+        """
+
+        url = ('/projects/%(project_id)s/access_requests/%(id)s/approve' %
+               {'project_id': self.project_id, 'id': self.id})
+        data = {'access_level': access_level}
+        r = self.gitlab._raw_put(url, data=data, **kwargs)
+        raise_error_from_response(r, GitlabUpdateError, 201)
+        self._set_from_dict(r.json())
+
+
+class ProjectAccessRequestManager(BaseManager):
+    obj_cls = ProjectAccessRequest
+
+
+class ProjectDeployment(GitlabObject):
+    _url = '/projects/%(project_id)s/deployments'
+    canCreate = False
+    canUpdate = False
+    canDelete = False
+
+
+class ProjectDeploymentManager(BaseManager):
+    obj_cls = ProjectDeployment
+
+
 class Project(GitlabObject):
     _url = '/projects'
     _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
@@ -1692,6 +1892,8 @@ class Project(GitlabObject):
                            'public_builds']
     shortPrintAttr = 'path'
     managers = [
+        ('accessrequests', ProjectAccessRequestManager,
+         [('project_id', 'id')]),
         ('branches', ProjectBranchManager, [('project_id', 'id')]),
         ('builds', ProjectBuildManager, [('project_id', 'id')]),
         ('commits', ProjectCommitManager, [('project_id', 'id')]),
@@ -1699,6 +1901,7 @@ class Project(GitlabObject):
          [('project_id', 'id')]),
         ('commit_statuses', ProjectCommitStatusManager,
          [('project_id', 'id')]),
+        ('deployments', ProjectDeploymentManager, [('project_id', 'id')]),
         ('environments', ProjectEnvironmentManager, [('project_id', 'id')]),
         ('events', ProjectEventManager, [('project_id', 'id')]),
         ('files', ProjectFileManager, [('project_id', 'id')]),
@@ -1711,114 +1914,17 @@ class Project(GitlabObject):
         ('mergerequests', ProjectMergeRequestManager, [('project_id', 'id')]),
         ('milestones', ProjectMilestoneManager, [('project_id', 'id')]),
         ('notes', ProjectNoteManager, [('project_id', 'id')]),
+        ('pipelines', ProjectPipelineManager, [('project_id', 'id')]),
+        ('services', ProjectServiceManager, [('project_id', 'id')]),
         ('snippets', ProjectSnippetManager, [('project_id', 'id')]),
         ('tags', ProjectTagManager, [('project_id', 'id')]),
         ('triggers', ProjectTriggerManager, [('project_id', 'id')]),
         ('variables', ProjectVariableManager, [('project_id', 'id')]),
     ]
 
-    VISIBILITY_PRIVATE = 0
-    VISIBILITY_INTERNAL = 10
-    VISIBILITY_PUBLIC = 20
-
-    def Branch(self, id=None, **kwargs):
-        warnings.warn("`Branch` is deprecated, use `branches` instead",
-                      DeprecationWarning)
-        return ProjectBranch._get_list_or_object(self.gitlab, id,
-                                                 project_id=self.id,
-                                                 **kwargs)
-
-    def Commit(self, id=None, **kwargs):
-        warnings.warn("`Commit` is deprecated, use `commits` instead",
-                      DeprecationWarning)
-        return ProjectCommit._get_list_or_object(self.gitlab, id,
-                                                 project_id=self.id,
-                                                 **kwargs)
-
-    def Event(self, id=None, **kwargs):
-        warnings.warn("`Event` is deprecated, use `events` instead",
-                      DeprecationWarning)
-        return ProjectEvent._get_list_or_object(self.gitlab, id,
-                                                project_id=self.id,
-                                                **kwargs)
-
-    def File(self, id=None, **kwargs):
-        warnings.warn("`File` is deprecated, use `files` instead",
-                      DeprecationWarning)
-        return ProjectFile._get_list_or_object(self.gitlab, id,
-                                               project_id=self.id,
-                                               **kwargs)
-
-    def Hook(self, id=None, **kwargs):
-        warnings.warn("`Hook` is deprecated, use `hooks` instead",
-                      DeprecationWarning)
-        return ProjectHook._get_list_or_object(self.gitlab, id,
-                                               project_id=self.id,
-                                               **kwargs)
-
-    def Key(self, id=None, **kwargs):
-        warnings.warn("`Key` is deprecated, use `keys` instead",
-                      DeprecationWarning)
-        return ProjectKey._get_list_or_object(self.gitlab, id,
-                                              project_id=self.id,
-                                              **kwargs)
-
-    def Issue(self, id=None, **kwargs):
-        warnings.warn("`Issue` is deprecated, use `issues` instead",
-                      DeprecationWarning)
-        return ProjectIssue._get_list_or_object(self.gitlab, id,
-                                                project_id=self.id,
-                                                **kwargs)
-
-    def Label(self, id=None, **kwargs):
-        warnings.warn("`Label` is deprecated, use `labels` instead",
-                      DeprecationWarning)
-        return ProjectLabel._get_list_or_object(self.gitlab, id,
-                                                project_id=self.id,
-                                                **kwargs)
-
-    def Member(self, id=None, **kwargs):
-        warnings.warn("`Member` is deprecated, use `members` instead",
-                      DeprecationWarning)
-        return ProjectMember._get_list_or_object(self.gitlab, id,
-                                                 project_id=self.id,
-                                                 **kwargs)
-
-    def MergeRequest(self, id=None, **kwargs):
-        warnings.warn(
-            "`MergeRequest` is deprecated, use `mergerequests` instead",
-            DeprecationWarning)
-        return ProjectMergeRequest._get_list_or_object(self.gitlab, id,
-                                                       project_id=self.id,
-                                                       **kwargs)
-
-    def Milestone(self, id=None, **kwargs):
-        warnings.warn("`Milestone` is deprecated, use `milestones` instead",
-                      DeprecationWarning)
-        return ProjectMilestone._get_list_or_object(self.gitlab, id,
-                                                    project_id=self.id,
-                                                    **kwargs)
-
-    def Note(self, id=None, **kwargs):
-        warnings.warn("`Note` is deprecated, use `notes` instead",
-                      DeprecationWarning)
-        return ProjectNote._get_list_or_object(self.gitlab, id,
-                                               project_id=self.id,
-                                               **kwargs)
-
-    def Snippet(self, id=None, **kwargs):
-        warnings.warn("`Snippet` is deprecated, use `snippets` instead",
-                      DeprecationWarning)
-        return ProjectSnippet._get_list_or_object(self.gitlab, id,
-                                                  project_id=self.id,
-                                                  **kwargs)
-
-    def Tag(self, id=None, **kwargs):
-        warnings.warn("`Tag` is deprecated, use `tags` instead",
-                      DeprecationWarning)
-        return ProjectTag._get_list_or_object(self.gitlab, id,
-                                              project_id=self.id,
-                                              **kwargs)
+    VISIBILITY_PRIVATE = gitlab.VISIBILITY_PRIVATE
+    VISIBILITY_INTERNAL = gitlab.VISIBILITY_INTERNAL
+    VISIBILITY_PUBLIC = gitlab.VISIBILITY_PUBLIC
 
     def tree(self, path='', ref_name='', **kwargs):
         warnings.warn("`tree` is deprecated, use `repository_tree` instead",
@@ -2142,7 +2248,7 @@ class RunnerManager(BaseManager):
 
         Raises:
             GitlabConnectionError: If the server cannot be reached.
-            GitlabListError; If the resource cannot be found
+            GitlabListError: If the resource cannot be found
         """
         url = '/runners/all'
         if scope is not None:
@@ -2158,6 +2264,33 @@ class TeamMember(GitlabObject):
     shortPrintAttr = 'username'
 
 
+class Todo(GitlabObject):
+    _url = '/todos'
+    canGet = 'from_list'
+    canUpdate = False
+    canCreate = False
+    optionalListAttrs = ['action', 'author_id', 'project_id', 'state', 'type']
+
+
+class TodoManager(BaseManager):
+    obj_cls = Todo
+
+    def delete_all(self, **kwargs):
+        """Mark all the todos as done.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabDeleteError: If the resource cannot be found
+
+        Returns:
+            The number of todos maked done.
+        """
+        url = '/todos'
+        r = self.gitlab._raw_delete(url, **kwargs)
+        raise_error_from_response(r, GitlabDeleteError)
+        return int(r.text)
+
+
 class UserProject(GitlabObject):
     _url = '/projects/user/%(user_id)s'
     _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
@@ -2269,20 +2402,6 @@ class Team(GitlabObject):
         ('projects', TeamProjectManager, [('team_id', 'id')])
     ]
 
-    def Member(self, id=None, **kwargs):
-        warnings.warn("`Member` is deprecated, use `members` instead",
-                      DeprecationWarning)
-        return TeamMember._get_list_or_object(self.gitlab, id,
-                                              team_id=self.id,
-                                              **kwargs)
-
-    def Project(self, id=None, **kwargs):
-        warnings.warn("`Project` is deprecated, use `projects` instead",
-                      DeprecationWarning)
-        return TeamProject._get_list_or_object(self.gitlab, id,
-                                               team_id=self.id,
-                                               **kwargs)
-
 
 class TeamManager(BaseManager):
     obj_cls = Team
